@@ -80,43 +80,38 @@ const HEIGHT: f64 = 500.0;
 
 trait PlotTrait<X: PlotNum, Y: PlotNum> {
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result;
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
+    fn iter_first<'a>(&'a mut self) -> Box<dyn Iterator<Item = (X, Y)> + 'a>;
+    fn iter_second<'a>(&'a mut self) -> Box<dyn Iterator<Item = (X, Y)> + 'a>;
 }
 
-struct PlotStruct<X: PlotNum, Y: PlotNum, I: Iterator<Item = (X, Y)> + Clone, F: Display> {
-    first: I,
-    second: I,
+struct PlotStruct<X: PlotNum, Y: PlotNum, I: PlotIter<Item = (X, Y)>, F: Display> {
+    iter: Option<I>,
     func: F,
     _p: PhantomData<(X, Y)>,
 }
 
-impl<X: PlotNum, Y: PlotNum, I: Iterator<Item = (X, Y)> + Clone, F: Display>
-    PlotStruct<X, Y, I, F>
-{
-    fn new(it: I, func: F) -> Self {
-        let it2 = it.clone();
+impl<X: PlotNum, Y: PlotNum, I: PlotIter<Item = (X, Y)>, F: Display> PlotStruct<X, Y, I, F> {
+    fn new(iter: I, func: F) -> Self {
         PlotStruct {
-            first: it,
-            second: it2,
+            iter: Some(iter),
             func,
             _p: PhantomData,
         }
     }
 }
 
-impl<X: PlotNum, Y: PlotNum, D: Iterator<Item = (X, Y)> + Clone, F: Display> PlotTrait<X, Y>
+impl<X: PlotNum, Y: PlotNum, D: PlotIter<Item = (X, Y)>, F: Display> PlotTrait<X, Y>
     for PlotStruct<X, Y, D, F>
 {
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result {
         write!(a, "{}", self.func)
     }
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
-        &mut self.first
+    fn iter_first<'a>(&'a mut self) -> Box<dyn Iterator<Item = (X, Y)> + 'a> {
+        Box::new(self.iter.as_mut().unwrap().first())
     }
 
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
-        &mut self.second
+    fn iter_second<'a>(&'a mut self) -> Box<dyn Iterator<Item = (X, Y)> + 'a> {
+        Box::new(self.iter.take().unwrap().second())
     }
 }
 
@@ -126,6 +121,7 @@ enum PlotType {
     Histo,
     LineFill,
     LineFillRaw,
+    Text,
 }
 
 struct Plot<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
@@ -282,6 +278,8 @@ pub fn data<'a, X: PlotNum, Y: PlotNum>() -> Data<'a, X, Y> {
     Data::default()
 }
 
+use plotnum::PlotIter;
+
 ///
 /// Plot collector.
 ///
@@ -314,18 +312,37 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
         self
     }
 
+    ///
+    /// Write some text in the legend. This doesnt increment the plot number.
+    ///
+    /// ```
+    /// let mut plotter = poloto::data::<f64,f64>();
+    /// plotter.text("This is a note");
+    /// ```
+    pub fn text(&mut self, name: impl Display + 'a) -> &mut Self {
+        self.plots.push(Plot {
+            plot_type: PlotType::Text,
+            plots: Box::new(PlotStruct::new(std::iter::empty(), name)),
+        });
+        self
+    }
+
+    /// Create a line from plots using a SVG path element.
+    /// The path element belongs to the `.poloto[N]fill` css class.
+    ///
+    /// ```
+    /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
+    /// let mut plotter = poloto::data();
+    /// plotter.line("", &data);
+    /// ```
     pub fn line<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
-        I: IntoIterator,
-        I::IntoIter: Clone + 'a,
+        I: PlotIter + 'a,
         I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Line,
-            plots: Box::new(PlotStruct::new(
-                plots.into_iter().map(|x| x.make_plot()),
-                name,
-            )),
+            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
         });
         self
     }
@@ -340,16 +357,12 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     /// ```
     pub fn line_fill<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
-        I: IntoIterator,
-        I::IntoIter: Clone + 'a,
+        I: PlotIter + 'a,
         I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::LineFill,
-            plots: Box::new(PlotStruct::new(
-                plots.into_iter().map(|x| x.make_plot()),
-                name,
-            )),
+            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
         });
         self
     }
@@ -365,16 +378,12 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     /// ```
     pub fn line_fill_raw<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
-        I: IntoIterator,
-        I::IntoIter: Clone + 'a,
+        I: PlotIter + 'a,
         I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::LineFillRaw,
-            plots: Box::new(PlotStruct::new(
-                plots.into_iter().map(|x| x.make_plot()),
-                name,
-            )),
+            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
         });
         self
     }
@@ -391,16 +400,12 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     /// ```
     pub fn scatter<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
-        I: IntoIterator,
-        I::IntoIter: Clone + 'a,
+        I: PlotIter + 'a,
         I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Scatter,
-            plots: Box::new(PlotStruct::new(
-                plots.into_iter().map(|x| x.make_plot()),
-                name,
-            )),
+            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
         });
         self
     }
@@ -416,16 +421,12 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     /// ```
     pub fn histogram<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
-        I: IntoIterator,
-        I::IntoIter: Clone + 'a,
+        I: PlotIter + 'a,
         I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Histo,
-            plots: Box::new(PlotStruct::new(
-                plots.into_iter().map(|x| x.make_plot()),
-                name,
-            )),
+            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
         });
         self
     }
